using System;
using System.Drawing;

using MonoTouch.Foundation;
using MonoTouch.UIKit;
using System.Net;
using System.IO;
using System.Json;
using HtmlAgilityPack;

using System.Collections.Generic;
namespace muss
{
	public partial class FirstViewController : UIViewController
	{
		public FirstViewController (IntPtr handle) : base (handle)
		{
			Title = NSBundle.MainBundle.LocalizedString ("First", "First");
			TabBarItem.Image = UIImage.FromBundle ("first");
		}
		
		public override void DidReceiveMemoryWarning ()
		{
			// Releases the view if it doesn't have a superview.
			base.DidReceiveMemoryWarning ();
			
			// Release any cached data, images, etc that aren't in use.
		}
		
		#region View lifecycle
		
		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();
			
			// Perform any additional setup after loading the view, typically from a nib.
			Title = "Football";



			var request = HttpWebRequest.Create("http://utahutes.cstv.com/sports/m-footbl/sched/utah-m-footbl-sched.html");
			request.ContentType = "html";
			request.Method = "GET";
			
			using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
			{
				if (response.StatusCode != HttpStatusCode.OK)
					Console.Out.WriteLine("Error fetching data. Server returned status code: {0}", response.StatusCode);
				using (StreamReader reader = new StreamReader(response.GetResponseStream()))
				{
					var content = reader.ReadToEnd();
					HtmlDocument doc  = new HtmlDocument();
					doc.LoadHtml(content);

					List<string> games =  new List<string>();
					List<string> dates = new List<string>();
					foreach(HtmlNode link in doc.DocumentNode.SelectNodes("//table[@id='schedtable']//tr[@valign='TOP']//td"))
					{

						//Console.WriteLine(link.InnerHtml);
						if(link.InnerHtml.StartsWith("at") || link.InnerHtml.StartsWith("vs"))
						{
							games.Add(link.InnerHtml);
						}
						if(link.InnerHtml[2] == '/')
						{
						 	dates.Add(link.InnerHtml);
						}
					}
					schedule.Source = new TableSource(games, dates);
				}
				
			}



		}
		
		public override void ViewWillAppear (bool animated)
		{
			base.ViewWillAppear (animated);
		}
		
		public override void ViewDidAppear (bool animated)
		{
			base.ViewDidAppear (animated);
		}
		
		public override void ViewWillDisappear (bool animated)
		{
			base.ViewWillDisappear (animated);
		}
		
		public override void ViewDidDisappear (bool animated)
		{
			base.ViewDidDisappear (animated);
		}
		
		#endregion
		
		public override bool ShouldAutorotateToInterfaceOrientation (UIInterfaceOrientation toInterfaceOrientation)
		{
			// Return true for supported orientations
			return (toInterfaceOrientation != UIInterfaceOrientation.PortraitUpsideDown);
		}
	}

	public class TableSource : UITableViewSource 
	{
		protected List<string> tableItems;
		protected List<string> itemDates;
		protected string cellIdentifier = "TableCell";
		public TableSource (List<string> items, List<string> dates)
		{
			tableItems = items;
			itemDates = dates;
		}
		public override int RowsInSection (UITableView tableview, int section)
		{
			return tableItems.Count;
		}
		public override UITableViewCell GetCell (UITableView tableView, MonoTouch.Foundation.NSIndexPath indexPath)
		{
			// request a recycled cell to save memory
			UITableViewCell cell = tableView.DequeueReusableCell (cellIdentifier);
			// if there are no cells to reuse, create a new one
			if (cell == null)
				cell = new UITableViewCell (UITableViewCellStyle.Subtitle, cellIdentifier);
			cell.TextLabel.Text = tableItems[indexPath.Row];
			cell.DetailTextLabel.Text = itemDates[indexPath.Row];
			return cell;
		}
		public override void RowSelected (UITableView tableView, NSIndexPath indexPath)
		{
			new UIAlertView("Row Selected", tableItems[indexPath.Row], null, "OK", null).Show();
			tableView.DeselectRow (indexPath, true); // iOS convention is to remove the highlight
		}
	}
}

